var clock/game_clock = new

world
	fps = 60
	maxx = 25
	maxy = 25
	turf = /turf/checker
	mob = /mob/player

// Checker pattern turfs.
turf/checker
	icon_state = "rect"
	color = "gray"
	New() color = (x + y) % 2 ? "#555" : "#525252"

mob/player
	icon_state = "oval"
	color = "blue"

	var tmp
		// Represents movement input.
		input_move_x = 0
		input_move_y = 0

		// Represents movement input that is smoothed over time.
		input_velocity_x = 0
		input_velocity_y = 0

		// Fastest you can control yourself, in pixels per frame.
		speed
		walk_speed = 5
		run_speed = 10

		// A factor that determines how quickly you go between zero and max_speed.
		speed_lerp = 0.2

	New()
		// Start ticking along with the game clock.
		EVENT_ADD(game_clock.OnTick, src, .proc/Update)
		speed = walk_speed

	Login()
		// Start listening for changes in input state.
		EVENT_ADD(client.ButtonPressed, src, .proc/HandleButtonPressed)
		EVENT_ADD(client.ButtonReleased, src, .proc/HandleButtonReleased)
		EVENT_ADD(client.MouseMoved, src, .proc/HandleMouseMoved)
		EVENT_ADD(client.AxisChanged, src, .proc/HandleAxisChanged)
		..()

	Logout()
		// Who needs garbage collection?
		del src

	Stat()
		stat("Mouse", "")
		var mouse_screen_position[] = client.GetMouseScreenPosition()
		stat("  Screen Position", mouse_screen_position \
			? jointext(mouse_screen_position, ", ") \
			: "N/A")

		var mouse_map_position[] = client.GetMouseMapPosition()
		stat("  Map Position", mouse_map_position \
			? jointext(mouse_map_position, ", ") \
			: "N/A")

	proc
		// Debug output of thumb stick input
		HandleAxisChanged(InputHandler/InputHandler, Axis, X, Y)
			world.log << "[Axis]: [X], [Y]"

			var left_axis[] = InputHandler.GetAxisValues(GamepadAxis.Left)
			world.log << "GamepadAxis.Left: [json_encode(left_axis)]"
			var right_axis[] = InputHandler.GetAxisValues(GamepadAxis.Right)
			world.log << "GamepadAxis.Right: [json_encode(right_axis)]"

		// A movement of the mouse was detected!
		HandleMouseMoved(InputHandler/InputHandler, MoveX, MoveY)
			// If the left mouse button is being pressed,
			var modifiers = InputHandler.GetModifierMask()
			world.log << "Mouse moved [MoveX], [MoveY] with modifiers [modifiers]"
			if(InputHandler.IsButtonPressed(MouseButton.Left) \
				&& modifiers == ModifierKeys.None)
				// Move in the opposite direction of the mouse's movement.
				// This results in the appearance of dragging the world around yourself.
				TranslateXY(-MoveX, -MoveY, TranslateFlags.EnableSliding)

		// A change in button state was detected!
		HandleButtonPressed(InputHandler/InputHandler, Button)
			world.log << "Button pressed: [Button]"
			switch(Button)
				if(KeyButton.D) input_move_x++
				if(KeyButton.A) input_move_x--
				if(KeyButton.W) input_move_y++
				if(KeyButton.S) input_move_y--
				if(KeyButton.Shift) speed = run_speed

		HandleButtonReleased(InputHandler/InputHandler, Button)
			world.log << "Button released: [Button]"
			switch(Button)
				if(KeyButton.D) input_move_x--
				if(KeyButton.A) input_move_x++
				if(KeyButton.W) input_move_y--
				if(KeyButton.S) input_move_y++
				if(KeyButton.Shift) speed = walk_speed

		Update(clock/Clock)
			input_velocity_x = lerp(input_velocity_x, input_move_x * speed, speed_lerp)
			input_velocity_y = lerp(input_velocity_y, input_move_y * speed, speed_lerp)
			/* Some notes about the above two lines:

				[input_move_x * max_speed] is either 0, [-max_speed], or [max_speed],
			because [input_move_x] is either 0, -1, or 1. Math!

				The [lerp()] here is to cause "exponential decay" of some value,
			[input_velocity_x], toward some target value, [input_move_x * max_speed].

				[speed_lerp] is a number between 0 and 1 that determines
			how quickly the value tends toward the target.

				The effect is that the player gradually accelerates and decelerates
			between 0, [-max_speed], and [max_speed], according to movement input.

			*/

			// Move along the velocity vector.
			TranslateXY(input_velocity_x, input_velocity_y, TranslateFlags.EnableSliding)

/*

	This is a proc that comes standard in probably every math library ever.
	lerp stands for linear interpolation.
	It basically gets you the number that is c% of the way from a to b.
	e.g. lerp(0, 100, 0.5) == 50, lerp(100, 200, 0.25) == 125.

*/
proc/lerp(a, b, c) return a * (1 - c) + b * c
